@@ -838,6 +838,7 @@ generate_reloc_directory()
 
 	/* Now create the . and .. entries in rr_moved */
 	/* Now create an actual directory  entry */
+	memset(&root_statbuf, 0x0, sizeof(struct stat)); /* be sure */
 	attach_dot_entries(reloc_dir, &root_statbuf);
 }
 
@@ -1546,17 +1547,23 @@ insert_file_entry(struct directory *this_dir, char *whole_path,
 #endif
 		return (0);
 	}
-	if (S_ISREG(lstatbuf.st_mode) && (lstatbuf.st_size >= (off_t)0x7FFFFFFF)) {
-		fprintf(stderr, "File %s is larger than 2GiB.\n", whole_path);
-		if(allow_limited_size) {
-			fprintf(stderr, "This size can only be represented in the UDF filesystem.\n"
-					"Make sure that your clients support and use it.\n"
-                                        "ISO9660, Joliet, RockRidge, HFS will display incorrect size.\n");
-		}
-		else {
+	/* print a warning but don't spam too much */
+	if (S_ISREG(lstatbuf.st_mode) && (lstatbuf.st_size >= (off_t)0xFFFFFFFF)) {
+		static int udf_warned;
+
+		if( !allow_limited_size || verbose>1)
+			fprintf(stderr, "File %s is larger than 4GiB-1.\n", whole_path);
+		if( !allow_limited_size)
+		{
 			fprintf(stderr, "-allow-limited-size was not specified. There is no way do represent this file size. Aborting.\n");
 			exit(1);
 		}
+		if(verbose>=1 && ! udf_warned ) {
+			udf_warned++;
+			fprintf(stderr, "This size can only be represented in the UDF filesystem.\n"
+					"Make sure that your clients support and use it.\n"
+					"ISO9660, Joliet, RockRidge, HFS will display incorrect size.\n");
+		}
 	}
 	/*
 	 * Add this so that we can detect directory loops with hard links.
@@ -1589,10 +1596,12 @@ insert_file_entry(struct directory *this_dir, char *whole_path,
 		!S_ISFIFO(lstatbuf.st_mode) && !S_ISSOCK(lstatbuf.st_mode) &&
 		!S_ISLNK(lstatbuf.st_mode) && !S_ISREG(lstatbuf.st_mode) &&
 		!S_ISDIR(lstatbuf.st_mode)) {
-		fprintf(stderr,
-		"Unknown file type (%s) %s - ignoring and continuing.\n",
-			filetype((int) lstatbuf.st_mode), whole_path);
-		return (0);
+        if ( ! (this_dir == root && strcmp(short_name, "..") == 0)) {
+            fprintf(stderr,
+                    "Unknown file type (%s) %s - ignoring and continuing.\n",
+                    filetype((int) lstatbuf.st_mode), whole_path);
+            return (0);
+        }
 	}
 	/* Who knows what trash this is - ignore and continue */
 
